/**********************************
 1D- and 2D-FFT of 2D Complex Data 
 H. Lee, 17/7/01 
 modified usgin ma.c, 7 May 2004
 Normalized.

   data: double complex in BIP
   nrow, ncol:

   iflag:  
	0 --- x directional FFT
	1 --- y directional FFT

   isign: (differ from definition to definition) 
            -1 --- e(-j2pixy)dx (Forward FFT) 
	  1 --- e(+j2pixy)dx (Inverse FFT) 
**********************************/

/*#include "ma.c"*/
#include "twon.c"
#include "dfour1.c"

void cfft1d(data, nrow, ncol, iflag, isign)
double *data;
int nrow, ncol, iflag, isign;
{
	unsigned long nn;
	int i,j, linen;
	double *tmp;

	linen = ncol*2; 
	
	/* x-directional FFT */ 
	if(iflag == 0)	 
	{
		tmp = dvector(ncol*2+1);
		nn = twon(ncol);
        	for(i=0; i<nrow; i++){
               	 	for(j=0; j<ncol; j++){
               	         	tmp[2*j+1] = data[linen*i + 2*j];
               		        tmp[2*j+2] = data[linen*i + 2*j + 1];
                	}
                	dfour1(tmp, nn, isign);
                	for(j=0; j<ncol; j++){
                        	data[linen*i+2*j] = tmp[2*j+1];
                        	data[linen*i+2*j+1] = tmp[2*j+2];
        		}
		}
		/* scaling */
		if(isign==1) for(i=0; i<nrow*ncol*2; i++) data[i] /= ncol;
	}

	/* y-directional FFT */ 
	if(iflag == 1) 
	{
		tmp = dvector(nrow*2+1);
		nn = twon(nrow);
		for(i=0; i< ncol; i++){	
			for(j=0; j<nrow; j++){
				tmp[2*j+1] = data[linen*j+2*i]; 
				tmp[2*j+2] = data[linen*j+2*i+1];
			}
			dfour1(tmp, nn, isign);
			for(j=0; j<nrow; j++){
				data[linen*j+2*i] = tmp[2*j+1];
				data[linen*j+2*i+1] = tmp[2*j+2];
			}
		}
		/* scaling */	
		if(isign==1) for(i=0; i<nrow*ncol*2; i++) data[i] /= nrow;
	}

	free(tmp);

}

void cfft2d(data, nrow, ncol, isign) 
double *data;
int nrow, ncol, isign;
{
	cfft1d(data, nrow, ncol, 0, isign);
	cfft1d(data, nrow, ncol, 1, isign);
}

void cfft1d_ri(realdata, imagdata, nrow, ncol, iflag, isign)  
double *realdata, *imagdata;
int nrow, ncol, iflag, isign;
{
	int i;
	double *data;
	
	data=dvector(nrow*ncol*2);

	for(i=0; i<nrow*ncol; i++) 
	{ 
		data[2*i]= realdata[i];
		data[2*i+1]=imagdata[i];
	}

	cfft1d(data, nrow, ncol, iflag, isign);
	
	for(i=0; i<nrow*ncol; i++) 
	{ 
		realdata[i]=data[2*i];
		imagdata[i]=data[2*i+1];
	}

	free(data);

}

void cfft2d_ri(realdata, imagdata, nrow, ncol, isign)
double *realdata, *imagdata;
int nrow, ncol, isign;
{
	cfft1d_ri(realdata, imagdata, nrow, ncol, 0, isign);
	cfft1d_ri(realdata, imagdata, nrow, ncol, 1, isign);
}


 

  
